﻿using UnityEngine;
using PixelCrushers.QuestMachine;
using PixelCrushers.QuestMachine.ORKSupport;

namespace ORKFramework.Events.Steps
{

    /// <summary>
    /// ORK event to get the number of quests held by a quest giver or player.
    /// </summary>
    [ORKEditorHelp("Get Quest Count", "Gets the number of quests held by a quest giver or player.", "")]
    [ORKEventStep(typeof(BaseEvent))]
    [ORKNodeInfo("Quest Machine/Quest Control")]
    public class GetQuestCountStep : BaseEventStep
    {

        [ORKEditorHelp("Quest List Container", "Select the quest list container, which can be a quest giver or a quest journal (e.g,. player).", "")]
        [ORKEditorInfo(labelText = "Quest List Container")]
        public EventObjectSetting qlcObject = new EventObjectSetting();

        // Result variable
        [ORKEditorHelp("Variable Origin", "Select the origin of the variables:\n" +
                       "- Local: Local variables are only used in a running event and don't interfere with global variables. " +
                       "The variable will be gone once the event ends.\n" +
                       "- Global: Global variables are persistent and available everywhere, everytime. " +
                       "They can be saved in save games.\n" +
                       "- Object: Object variables are bound to objects in the scene by an object ID. " +
                       "They can be saved in save games.", "")]
        public VariableOrigin origin = VariableOrigin.Global;

        // object variables
        [ORKEditorHelp("Use Object", "Use the 'Object Variables' component of game objects to change the object variables.\n" +
                       "The changes will be made on every 'Object Variables' component that is found. " +
                       "If no component is found, no variables will be changed.\n" +
                       "If disabled, you need to define the object ID used to change the object variables.", "")]
        [ORKEditorInfo(separator = true)]
        [ORKEditorLayout("origin", VariableOrigin.Object)]
        public bool useObject = true;

        [ORKEditorInfo(separator = true, labelText = "Object")]
        [ORKEditorLayout("useObject", true, autoInit = true)]
        public EventObjectSetting variableObject;

        [ORKEditorHelp("Object ID", "Define the object ID of the object variables.\n" +
                       "If the object ID doesn't exist yet, it will be created.", "")]
        [ORKEditorInfo(expandWidth = true)]
        [ORKEditorLayout(elseCheckGroup = true, endCheckGroup = true, endGroups = 2)]
        public string objectID = "";

        // variable key
        [ORKEditorHelp("Variable Key", "The key of the variable that will hold the return value.\n" +
                       "Typically you'll set the Value Type to Value and enter the name of the variable's key.", "")]
        [ORKEditorInfo(separator = true, labelText = "Variable Key")]
        public StringValue key = new StringValue();

        [ORKEditorHelp("Variable Type", "The return value type.\n" +
                       "Float and String are allowed. Others are not, and will be converted to a string.", "")]
        [ORKEditorInfo(separator = false, labelText = "Variable Type")]
        public GameVariableType variableType = GameVariableType.String;

        public override void Execute(BaseEvent baseEvent)
        {
            var list = qlcObject.GetObject(baseEvent);
            var questListContainer = ((list != null) && (list.Count > 0)) ? list[0].transform.GetComponent<QuestListContainer>() : null;
            if (questListContainer == null)
            {
                Debug.LogWarning("Quest Machine: Can't find quest list container to get quest count.");
            }
            else
            {
                ORKUtility.SetVariableValue(baseEvent, questListContainer.questList.Count, origin, useObject, variableObject, objectID, key);
            }
            baseEvent.StepFinished(this.next);
        }

    }
}
