﻿using UnityEngine;
using PixelCrushers.QuestMachine;
using PixelCrushers.QuestMachine.ORKSupport;

namespace ORKFramework.Events.Steps
{

    /// <summary>
    /// ORK event to get a quest counter value.
    /// </summary>
    [ORKEditorHelp("Get Quest Counter", "Gets a quest counter value.", "")]
    [ORKEventStep(typeof(BaseEvent))]
    [ORKNodeInfo("Quest Machine/Quest Control")]
    public class GetQuestCounterStep : BaseEventStep
    {

        [ORKEditorHelp("Quest ID", "Set the counter in this quest.", "")]
        [ORKEditorInfo(labelText = "Quest ID")]
        public StringValue questID = new StringValue();

        [ORKEditorHelp("Counter Name", "Set this counter.", "")]
        [ORKEditorInfo(labelText = "Counter Name")]
        public StringValue counterName = new StringValue();

        // Result variable
        [ORKEditorHelp("Variable Origin", "Select the origin of the variables:\n" +
                       "- Local: Local variables are only used in a running event and don't interfere with global variables. " +
                       "The variable will be gone once the event ends.\n" +
                       "- Global: Global variables are persistent and available everywhere, everytime. " +
                       "They can be saved in save games.\n" +
                       "- Object: Object variables are bound to objects in the scene by an object ID. " +
                       "They can be saved in save games.", "")]
        public VariableOrigin origin = VariableOrigin.Global;

        // object variables
        [ORKEditorHelp("Use Object", "Use the 'Object Variables' component of game objects to change the object variables.\n" +
                       "The changes will be made on every 'Object Variables' component that is found. " +
                       "If no component is found, no variables will be changed.\n" +
                       "If disabled, you need to define the object ID used to change the object variables.", "")]
        [ORKEditorInfo(separator = true)]
        [ORKEditorLayout("origin", VariableOrigin.Object)]
        public bool useObject = true;

        [ORKEditorInfo(separator = true, labelText = "Object")]
        [ORKEditorLayout("useObject", true, autoInit = true)]
        public EventObjectSetting variableObject;

        [ORKEditorHelp("Object ID", "Define the object ID of the object variables.\n" +
                       "If the object ID doesn't exist yet, it will be created.", "")]
        [ORKEditorInfo(expandWidth = true)]
        [ORKEditorLayout(elseCheckGroup = true, endCheckGroup = true, endGroups = 2)]
        public string objectID = "";

        // variable key
        [ORKEditorHelp("Variable Key", "The key of the variable that will hold the return value.\n" +
                       "Typically you'll set the Value Type to Value and enter the name of the variable's key.", "")]
        [ORKEditorInfo(separator = true, labelText = "Variable Key")]
        public StringValue key = new StringValue();

        [ORKEditorHelp("Variable Type", "The return value type.\n" +
                       "Float and String are allowed. Others are not, and will be converted to a string.", "")]
        [ORKEditorInfo(separator = false, labelText = "Variable Type")]
        public GameVariableType variableType = GameVariableType.String;

        public override void Execute(BaseEvent baseEvent)
        {
            var quest = QuestMachine.GetQuestInstance(questID.GetValue());
            if (quest == null)
            {
                Debug.LogWarning("Quest Machine: Can't find quest '" + questID + "' to get counter " + counterName + " value.");
            }
            else
            {
                var counter = quest.GetCounter(counterName.GetValue());
                if (counter == null)
                {
                    Debug.LogWarning("Quest Machine: Can't find a counter named '" + counterName + "' in quest '" + questID + "' .");
                }
                else
                {
                    ORKUtility.SetVariableValue(baseEvent, counter.currentValue, origin, useObject, variableObject, objectID, key);
                }
            }
            baseEvent.StepFinished(this.next);
        }

    }
}
