﻿// Copyright © Pixel Crushers. All rights reserved.

using UnityEngine;
using ORKFramework;

namespace PixelCrushers.QuestMachine.ORKSupport
{

    /// <summary>
    /// Quest condition that checks an ORK variable value.
    /// </summary>
    public class ORKVariableQuestCondition : QuestCondition, IQuestTimer
    {
        [SerializeField]
        private bool m_globalVariable = true;

        [SerializeField]
        private StringField m_objectID = new StringField();

        [SerializeField]
        private StringField m_key = new StringField();

        [SerializeField]
        private ORKUtilityVariableType m_variableType;

        [SerializeField]
        private ORKUtilityBoolValue m_requiredBoolValue;

        [SerializeField]
        private CounterValueConditionMode m_criterion = CounterValueConditionMode.AtLeast;

        [SerializeField]
        private QuestNumber m_requiredIntValue = new QuestNumber(1);

        public bool globalVariable
        {
            get { return m_globalVariable; }
            set { m_globalVariable = value; }
        }

        public StringField objectID
        {
            get { return m_objectID; }
            set { m_objectID = value; }
        }

        public StringField key
        {
            get { return m_key; }
            set { m_key = value; }
        }

        public ORKUtilityVariableType variableType
        {
            get { return m_variableType; }
            set { m_variableType = value; }
        }

        public ORKUtilityBoolValue requiredBoolValue
        {
            get { return m_requiredBoolValue; }
            set { m_requiredBoolValue = value; }
        }

        public CounterValueConditionMode criterion
        {
            get { return m_criterion; }
            set { m_criterion = value; }
        }

        public QuestNumber requiredIntValue
        {
            get { return m_requiredIntValue; }
            set { m_requiredIntValue = value; }
        }

        private VariableHandler variableHandler
        {
            get { return globalVariable ? ORK.Game.Variables : ORK.Game.Scene.GetObjectVariables(StringField.GetStringValue(objectID)); }
        }

        public override string GetEditorName()
        {
            var s = "ORK " + (globalVariable ? "global" : StringField.GetStringValue(objectID)) + " " + key;
            switch (variableType)
            {
                case ORKUtilityVariableType.Bool:
                    return s + " is " + requiredBoolValue;
                case ORKUtilityVariableType.Int:
                    return s + " " + criterion + " " + requiredIntValue.literalValue;
                default:
                    return s;
            }
        }

        public override void StartChecking(System.Action trueAction)
        {
            base.StartChecking(trueAction);
            if (IsConditionMet())
            {
                SetTrue();
            }
            else
            {
                QuestTimerManager.RegisterTimer(this);
            }
        }

        public override void StopChecking()
        {
            base.StopChecking();
            QuestTimerManager.UnregisterTimer(this);
        }

        public void Tick()
        {
            if (IsConditionMet())
            {
                StopChecking();
                SetTrue();
            }
        }

        private bool IsConditionMet()
        {
            switch (variableType)
            {
                case ORKUtilityVariableType.Bool:
                    var boolValue = ORKUtility.GetVariableBoolValue(variableHandler, StringField.GetStringValue(key));
                    switch (requiredBoolValue)
                    {
                        case ORKUtilityBoolValue.False:
                        default:
                            return boolValue == false;
                        case ORKUtilityBoolValue.True:
                            return boolValue == true;
                    }
                case ORKUtilityVariableType.Int:
                    var intValue = ORKUtility.GetVariableIntValue(variableHandler, StringField.GetStringValue(key));
                    switch (criterion)
                    {
                        default:
                        case CounterValueConditionMode.AtLeast:
                            return intValue >= requiredIntValue.GetValue(quest);
                        case CounterValueConditionMode.AtMost:
                            return intValue <= requiredIntValue.GetValue(quest);
                    }
                default:
                    return false;
            }
        }

    }

}
