﻿// Copyright © Pixel Crushers. All rights reserved.

using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using ORKFramework;

namespace PixelCrushers.QuestMachine.ORKSupport
{

    /// <summary>
    /// Updates quest counters with ORK inventory counts. It's unidirectional; 
    /// when an inventory count changes, it updates the counter value. But if you
    /// manually change the counter value, it will not update ORK's inventory count.
    /// </summary>
    public class ORKInventoryCounterSynchronizer : ORKInventoryMonitor
    {

        protected override string purpose { get { return "monitor player's inventory to sync player inventory counts to quest counters"; } }

        protected Dictionary<string, int> itemIDs = null;
        protected Dictionary<string, int> weaponIDs = null;
        protected Dictionary<string, int> armorIDs = null;
        protected List<string> monitoredItems = new List<string>();

        protected override void OnInitialize()
        {
            base.OnInitialize();
            RefreshCounterItemNames();
        }

        protected void GetItemIDs()
        {
            var debugString = "Quest Machine: Items that can be monitored for quest counters:";
            var updated = false;

            if (itemIDs == null)
            {
                itemIDs = new Dictionary<string, int>();
                for (int i = 0; i < ORK.Items.Count; i++)
                {
                    var item = ORK.Items.Get(i);
                    if (item == null) continue;
                    var itemName = item.GetName();
                    if (string.IsNullOrEmpty(itemName)) continue;
                    if (itemIDs.ContainsKey(itemName))
                    {
                        if (Debug.isDebugBuild) Debug.LogWarning("Quest Machine: Encountered duplicate item name: '" + itemName + "'.", this);
                    }
                    else
                    {
                        itemIDs.Add(itemName, item.ID);
                        debugString += "\nItem ID=" + item.ID + ", Name=" + item.GetName();
                    }
                }
                updated = true;
            }
            if (weaponIDs == null)
            {
                weaponIDs = new Dictionary<string, int>();
                for (int i = 0; i < ORK.Weapons.Count; i++)
                {
                    var weapon = ORK.Weapons.Get(i);
                    if (weapon == null) continue;
                    var weaponName = weapon.GetName();
                    if (string.IsNullOrEmpty(weaponName)) continue;
                    if (weaponIDs.ContainsKey(weaponName))
                    {
                        if (Debug.isDebugBuild) Debug.LogWarning("Quest Machine: Encountered duplicate weapon name: '" + weaponName + "'.", this);
                    }
                    else
                    {
                        weaponIDs.Add(weaponName, weapon.ID);
                        debugString += "\nWeapon ID=" + weapon.ID + ", Name=" + weapon.GetName();
                    }
                }
                updated = true;
            }
            if (armorIDs == null)
            {
                armorIDs = new Dictionary<string, int>();
                for (int i = 0; i < ORK.Armors.Count; i++)
                {
                    var armor = ORK.Armors.Get(i);
                    if (armor == null) continue;
                    var armorName = armor.GetName();
                    if (string.IsNullOrEmpty(armorName)) continue;
                    if (armorIDs.ContainsKey(armorName))
                    {
                        if (Debug.isDebugBuild) Debug.LogWarning("Quest Machine: Encountered duplicate armor name: '" + armorName + "'.", this);
                    }
                    else
                    {
                        armorIDs.Add(armorName, armor.ID);
                        debugString += "\nArmor ID=" + armor.ID + ", Name=" + armor.GetName();
                    }
                }
                updated = true;
            }

            if (updated && QuestMachine.debug) Debug.Log(debugString);
        }

        protected virtual void RefreshCounterItemNames()
        {
            GetItemIDs();

            // Refresh counters that need monitoring and register for quest list changes:
            if (player == null || player.GameObject == null) return;
            var questListContainer = player.GameObject.GetComponent<QuestListContainer>();
            if (questListContainer == null || questListContainer.questList == null) return;

            questListContainer.questAdded -= OnQuestListChange;
            questListContainer.questAdded += OnQuestListChange;
            questListContainer.questRemoved -= OnQuestListChange;
            questListContainer.questRemoved += OnQuestListChange;

            monitoredItems.Clear();
            foreach (var quest in questListContainer.questList)
            {
                if (quest == null) continue;
                foreach (var counter in quest.counterList)
                {
                    if (counter == null) continue;
                    if (counter.updateMode == QuestCounterUpdateMode.DataSync)
                    {
                        var itemName = StringField.GetStringValue(counter.name);
                        if (string.IsNullOrEmpty(itemName)) continue;
                        if (!(itemIDs.ContainsKey(itemName) || weaponIDs.ContainsKey(itemName) || armorIDs.ContainsKey(itemName)))
                        {
                            if (Debug.isDebugBuild) Debug.LogWarning("Quest Machine: Quest '" + quest.id + "' on " + name + " has a data sync counter named '" + itemName + "' but no item with this name is defined in ORK. If you're syncing the value from another source, you can ignore this message.", this);
                        }
                        else
                        {
                            if (QuestMachine.debug) Debug.Log("Quest Machine: Will synchronize inventory count of item '" + itemName + "' to quest counter in '" + quest.id + "' on " + name + ".", this);
                            if (!monitoredItems.Contains(itemName))
                            {
                                monitoredItems.Add(itemName);
                            }
                        }
                    }
                }
            }
        }

        protected virtual void OnQuestListChange(Quest quest)
        {
            if (gameObject.activeInHierarchy && enabled)
            { 
                RefreshCounterItemNames();
                StartCoroutine(UpdateCountersAfterOneFrame());
            }
        }

        protected IEnumerator UpdateCountersAfterOneFrame()
        {
            OnInventoryChanged();
            // Refresh again after counters have registered for data sync messages:
            yield return null;
            OnInventoryChanged();
        }

        protected override void OnInventoryChanged()
        {
            if (playerInventory == null)
            {
                if (Debug.isDebugBuild) Debug.LogWarning("Quest Machine: " + name + " received a player inventory changed event from ORK, but " + name + " can't find the player inventory.", this);
                return;
            }

            var itemCounts = new Dictionary<string, int>();

            // Sum items in player's inventory and equipment:
            foreach (var itemName in monitoredItems)
            {
                if (!itemCounts.ContainsKey(itemName))
                {
                    itemCounts.Add(itemName, 0);
                }
                if (itemIDs.ContainsKey(itemName))
                {
                    var itemID = itemIDs[itemName];
                    itemCounts[itemName] += playerInventory.Items.GetCount(itemID);
                }
                if (weaponIDs.ContainsKey(itemName))
                {
                    var weaponID = weaponIDs[itemName];
                    itemCounts[itemName] += playerInventory.Weapons.GetCount(weaponID);
                }
                if (armorIDs.ContainsKey(itemName))
                {
                    var armorID = armorIDs[itemName];
                    itemCounts[itemName] += playerInventory.Armors.GetCount(armorID);
                }
            }

            // Sync to quests:
            foreach (var itemName in monitoredItems)
            {
                var count = itemCounts[itemName];
                if (QuestMachine.debug && player != null && player.GameObject != null) Debug.Log("Quest Machine: Syncing inventory count of " + count + " " + itemName + " held by " + player.GameObject.name + " to quest counters on " + name, this);
                MessageSystem.SendMessage(this, DataSynchronizer.DataSourceValueChangedMessage, itemName, count);
            }
        }
    }
}
