﻿// Copyright © Pixel Crushers. All rights reserved.

using UnityEngine;
using ORKFramework;
using ORKFramework.Behaviours;

namespace PixelCrushers.QuestMachine.ORKSupport
{

    /// <summary>
    /// Forwards QuestListContainer events to ORK event interactions.
    /// </summary>
    [AddComponentMenu("Pixel Crushers/Quest Machine/Third Party/ORK Framework/Quest List Events to ORK")]
    [RequireComponent(typeof(QuestListContainer))]
    public class QuestListEventsToORK : MonoBehaviour
    {

        [Tooltip("Store the quest ID in this ORK global variable.")]
        public StringField questIDGlobalVariable = new StringField();

        [Tooltip("Run this event when a quest is added.")]
        public EventInteraction questAddedEvent;

        [Tooltip("Run this event when a quest is removed.")]
        public EventInteraction questRemovedEvent;

        [Tooltip("Run this event when a quest becomes offerable.")]
        public EventInteraction questOfferableEvent;

        [Tooltip("Run this event when a quest's state changes.")]
        public EventInteraction questStateChangedEvent;

        [Tooltip("Run this event when a quest's node state changes.")]
        public EventInteraction questNodeStateChangedEvent;

        [Tooltip("Log debug info.")]
        public bool debug = false;

        private QuestListContainer m_questListContainer = null;
        private QuestListContainer questListContainer
        {
            get
            {
                if (m_questListContainer == null) m_questListContainer = GetComponent<QuestListContainer>();
                return m_questListContainer;
            }
        }

        private StringValue m_questIDKey = null;
        private StringValue questIDKey
        {
            get
            {
                if (m_questIDKey == null) m_questIDKey = new StringValue(StringField.GetStringValue(questIDGlobalVariable));
                return m_questIDKey;
            }
        }


        private void OnEnable()
        {
            if (questListContainer == null) return;
            questListContainer.forwardEventsToListeners = true;
            questListContainer.questAdded += OnQuestAdded;
            questListContainer.questRemoved += OnQuestRemoved;
            questListContainer.questBecameOfferable += OnQuestBecameOfferable;
            questListContainer.questStateChanged += OnQuestStateChanged;
            questListContainer.questNodeStateChanged += OnQuestNodeStateChanged;
        }

        private void OnDisable()
        {
            if (questListContainer == null) return;
            questListContainer.questAdded -= OnQuestAdded;
            questListContainer.questRemoved -= OnQuestRemoved;
            questListContainer.questBecameOfferable -= OnQuestBecameOfferable;
            questListContainer.questStateChanged -= OnQuestStateChanged;
            questListContainer.questNodeStateChanged -= OnQuestNodeStateChanged;
        }

        private void OnQuestAdded(Quest quest)
        {
            StartOrkEvent(questAddedEvent, quest);
        }

        private void OnQuestRemoved(Quest quest)
        {
            StartOrkEvent(questRemovedEvent, quest);
        }

        private void OnQuestBecameOfferable(Quest quest)
        {
            StartOrkEvent(questOfferableEvent, quest);
        }

        private void OnQuestStateChanged(Quest quest)
        {
            StartOrkEvent(questStateChangedEvent, quest);
        }

        private void OnQuestNodeStateChanged(QuestNode questNode)
        {
            StartOrkEvent(questNodeStateChangedEvent, questNode.quest);
        }

        public void StartOrkEvent(EventInteraction eventInteraction, Quest quest)
        {
            if (eventInteraction == null) return;
            if (quest != null && !StringField.IsNullOrEmpty(questIDGlobalVariable))
            {
                ORKUtility.SetVariableValue(ORK.Game.Variables, StringField.GetStringValue(quest.id), questIDKey);
            }
            eventInteraction.StartEvent(gameObject);
        }


    }
}
