﻿// Copyright © Pixel Crushers. All rights reserved.

using UnityEngine;
using ORKFramework.Behaviours;

namespace PixelCrushers.QuestMachine.ORKSupport
{
    /// <summary>
    /// Add this component to the player prefab to block player input when Quest Machine 
    /// sends "Pause Player" and "Unpause Player" messages, such as when opening and 
    /// closing the dialogue UI. You can also assign events to execute on these messages.
    /// </summary>
    public class ORKEventsOnPausePlayer : MonoBehaviour, IMessageHandler
    {
        [Tooltip("Block player input when Quest Machine sends 'Pause Player' message and unblock on 'Unpause Player' message.")]
        public bool blockPlayerInput = true;

        [Tooltip("Run this event when Quest Machine sends the 'Pause Player' message (e.g., dialogue start).")]
        public EventInteraction pausePlayerEvent;

        [Tooltip("Run this event when Quest Machine sends the 'Unpause Player' message (e.g., dialogue end).")]
        public EventInteraction unpausePlayerEvent;

        protected virtual void OnEnable()
        {
            MessageSystem.AddListener(this, "Pause Player", string.Empty);
            MessageSystem.AddListener(this, "Unpause Player", string.Empty);
        }

        protected virtual void OnDisable()
        {
            MessageSystem.RemoveListener(this);
        }

        public void OnMessage(MessageArgs messageArgs)
        {
            switch (messageArgs.message)
            {
                case "Pause Player":
                    if (blockPlayerInput)
                    {
                        SetComponentEnabled<ButtonPlayerController>(false);
                        SetComponentEnabled<MousePlayerController>(false);
                    }
                    if (pausePlayerEvent != null)
                    {
                        pausePlayerEvent.StartEvent(gameObject);
                    }
                    break;
                case "Unpause Player":
                    if (blockPlayerInput)
                    {
                        SetComponentEnabled<ButtonPlayerController>(true);
                        SetComponentEnabled<MousePlayerController>(true);
                    }
                    if (unpausePlayerEvent != null)
                    {
                        unpausePlayerEvent.StartEvent(gameObject);
                    }
                    break;
            }
        }

        protected void SetComponentEnabled<T>(bool value) where T : MonoBehaviour
        {
            T component = GetComponent<T>();
            if (component != null)
            {
                component.enabled = value;
            }
        }

    }
}
