﻿// Copyright © Pixel Crushers. All rights reserved.

using UnityEngine;
using System.Collections.Generic;
using ORKFramework;
using ORKFramework.Behaviours;
using ORKFramework.Events;

namespace PixelCrushers.QuestMachine.ORKSupport
{

    public enum ORKUtilityVariableType { Bool, Int }

    public enum ORKUtilityBoolValue { True, False }

    public enum ORKAddRemoveOperation { Add, Remove }

    /// <summary>
    /// Utility methods to interface Quest Machine with ORK Framework.
    /// </summary>
    public static class ORKUtility
    {

        #region Variables

        public static void SetVariableValue(BaseEvent baseEvent, System.Object value,
                                            VariableOrigin origin, bool useObject,
                                            EventObjectSetting variableObject, string objectID,
                                            StringValue key)
        {
            if (value is string || value is bool || value is int || value is float || value is Vector3)
            {
                if (VariableOrigin.Local.Equals(origin))
                {
                    SetVariableValue(baseEvent.Variables, value, key);
                }
                else if (VariableOrigin.Global.Equals(origin))
                {
                    SetVariableValue(ORK.Game.Variables, value, key);
                }
                else if (VariableOrigin.Object.Equals(origin))
                {
                    if (useObject)
                    {
                        List<GameObject> list2 = variableObject.GetObject(baseEvent);
                        for (int j = 0; j < list2.Count; j++)
                        {
                            if (list2[j] != null)
                            {
                                ObjectVariablesComponent[] comps = list2[j].
                                    GetComponentsInChildren<ObjectVariablesComponent>();
                                for (int k = 0; k < comps.Length; k++)
                                {
                                    SetVariableValue(comps[k].GetHandler(), value, key);
                                }
                            }
                        }
                    }
                    else
                    {
                        SetVariableValue(ORK.Game.Scene.GetObjectVariables(objectID), value, key);
                    }
                }
            }
        }

        public static void SetVariableValue(VariableHandler handler, System.Object value, StringValue key)
        {
            if (handler == null) return;
            if (value is string)
            {
                handler.Set(key.GetValue(), (string)value);
            }
            else if (value is bool)
            {
                handler.Set(key.GetValue(), (bool)value);
            }
            else if (value is int)
            {
                handler.Set(key.GetValue(), (int)value);
            }
            else if (value is float)
            {
                handler.Set(key.GetValue(), (float)value);
            }
            else if (value is Vector3)
            {
                handler.Set(key.GetValue(), (Vector3)value);
            }
        }

        public static bool GetVariableBoolValue(VariableHandler handler, string key)
        {
            return (handler != null) ? handler.GetBool(key) : false;
        }

        public static float GetVariableFloatValue(VariableHandler handler, string key)
        {
            return (handler != null) ? handler.GetFloat(key) : 0;
        }

        public static int GetVariableIntValue(VariableHandler handler, string key)
        {
            return (int)GetVariableFloatValue(handler, key);
        }

        #endregion

        #region Combatants

        public static Combatant GetCombatant(string combatantName)
        {
            if (string.IsNullOrEmpty(combatantName) || string.Equals(combatantName, "player", System.StringComparison.OrdinalIgnoreCase))
            {
                return ORK.Game.ActiveGroup.Leader;
            }
            else
            {
                return ComponentHelper.GetCombatant(FindGameObject(combatantName));
            }
        }

        private static GameObject FindGameObject(string gameObjectName)
        {
            var go = GameObjectUtility.GameObjectHardFind(gameObjectName);
            if (go == null) go = GameObjectUtility.GameObjectHardFind(gameObjectName + "(Clone)");
            return go;
        }

        #endregion

        #region Inventory and Currency

        public static bool ORKHasItemQuantity(string combatantName, string itemName, int quantity)
        {
            return ORKHasItemQuantity(GetCombatant(combatantName), itemName, quantity);
        }

        public static bool ORKHasItemQuantity(Combatant combatant, string itemName, int quantity)
        {
            if (combatant == null) return false;

            // Check items:
            for (int i = 0; i < ORK.Items.Count; i++)
            {
                if (string.Equals(ORK.Items.GetName(i), itemName))
                {
                    return combatant.Inventory.Has(new ItemShortcut(i, 1), quantity);
                }
            }
            // Check weapons:
            for (int i = 0; i < ORK.Weapons.Count; i++)
            {
                if (string.Equals(ORK.Weapons.GetName(i), itemName))
                {
                    return combatant.Inventory.Weapons.HasEquip(i, -1, quantity);
                }
            }
            // Check armor:
            for (int i = 0; i < ORK.Armors.Count; i++)
            {
                if (string.Equals(ORK.Armors.GetName(i), itemName))
                {
                    return combatant.Inventory.Armors.HasEquip(i, -1, quantity);
                }
            }
            return false;
        }

        public static void ORKAddItemQuantity(string combatantName, string itemName, int quantity)
        {
            var combatant = GetCombatant(combatantName);
            if (combatant == null) return;

            // Try adding item:
            for (int i = 0; i < ORK.Items.Count; i++)
            {
                if (string.Equals(ORK.Items.GetName(i), itemName))
                {
                    combatant.Inventory.Add(new ItemShortcut(i, quantity), true, true, true);
                    return;
                }
            }
            // Try adding weapon:
            for (int i = 0; i < ORK.Weapons.Count; i++)
            {
                if (string.Equals(ORK.Weapons.GetName(i), itemName))
                {
                    combatant.Inventory.Weapons.Add(new EquipShortcut(EquipSet.Weapon, i, 1, quantity), true, true, true);
                    return;
                }
            }
            // Try adding armor:
            for (int i = 0; i < ORK.Armors.Count; i++)
            {
                if (string.Equals(ORK.Armors.GetName(i), itemName))
                {
                    combatant.Inventory.Armors.Add(new EquipShortcut(EquipSet.Armor, i, 1, quantity), true, true, true);
                    return;
                }
            }
        }

        public static void ORKRemoveItemQuantity(string combatantName, string itemName, int quantity)
        {
            var combatant = GetCombatant(combatantName);
            if (combatant == null) return;
            // Try removing item:
            for (int i = 0; i < ORK.Items.Count; i++)
            {
                if (string.Equals(ORK.Items.GetName(i), itemName))
                {
                    combatant.Inventory.Remove(new ItemShortcut(i, 1), quantity, true, true);
                    return;
                }
            }
            // Try removing weapon:
            for (int i = 0; i < ORK.Weapons.Count; i++)
            {
                if (string.Equals(ORK.Weapons.GetName(i), itemName))
                {
                    combatant.Inventory.Remove(new EquipShortcut(EquipSet.Weapon, i, 1, 1), quantity, true, true);
                    return;
                }
            }
            // Try removing armor:
            for (int i = 0; i < ORK.Armors.Count; i++)
            {
                if (string.Equals(ORK.Armors.GetName(i), itemName))
                {
                    combatant.Inventory.Remove(new EquipShortcut(EquipSet.Armor, i, 1, 1), quantity, true, true);
                    return;
                }
            }
        }

        public static int ORKGetMoney(string combatantName, string currencyName)
        {
            return ORKGetMoney(GetCombatant(combatantName), GetCurrencyIndex(currencyName));
        }

        public static int ORKGetMoney(Combatant combatant, int currencyIndex)
        {
            if (combatant == null) return 0;
            return combatant.Inventory.GetMoney(currencyIndex);
        }

        public static void ORKSetMoney(string combatantName, string currencyName, int quantity)
        {
            var combatant = GetCombatant(combatantName);
            if (combatant == null) return;
            combatant.Inventory.SetMoney(GetCurrencyIndex(currencyName), quantity, true, true);
        }

        public static int GetCurrencyIndex(string currencyName)
        {
            for (int i = 0; i < ORK.Currencies.Count; i++)
            {
                if (string.Equals(ORK.Currencies.GetName(i), currencyName))
                {
                    return i;
                }
            }
            if (Debug.isDebugBuild) Debug.LogWarning("Quest Machine: Couldn't find ORK currency index for currency named '" + currencyName + "'");
            return 0;
        }

        #endregion

        #region Faction

        public static void ORKChangeFaction(string combatantName, string newFactionName)
        {
            var combatant = GetCombatant(combatantName);
            combatant.Group.FactionID = GetFactionID(newFactionName);
        }

        public static float ORKGetFactionSympathy(string factionName, string otherName)
        {
            return ORKGetFactionSympathy(GetFactionID(factionName), GetFactionID(otherName));
        }

        public static float ORKGetFactionSympathy(int factionID, int otherID)
        {
            return ORK.Game.Faction.GetSympathy(factionID, otherID);
        }

        public static void ORKSetFactionSympathy(string factionName, string otherName, float amount)
        {
            ORKChangeFactionSympathy(factionName, otherName, amount, SimpleOperator.Set);
        }

        public static void ORKAddFactionSympathy(string factionName, string otherName, float amount)
        {
            ORKChangeFactionSympathy(factionName, otherName, amount, SimpleOperator.Add);
        }

        public static void ORKSubFactionSympathy(string factionName, string otherName, float amount)
        {
            ORKChangeFactionSympathy(factionName, otherName, amount, SimpleOperator.Sub);
        }

        public static void ORKChangeFactionSympathy(string factionName, string otherName, float change, SimpleOperator op)
        {
            ORK.Game.Faction.ChangeSympathy(GetFactionID(factionName), GetFactionID(otherName), change, op);
        }

        public static int GetFactionID(string factionName)
        {
            for (int i = 0; i < ORK.Factions.Count; i++)
            {
                if (string.Equals(ORK.Factions.GetName(i), factionName))
                {
                    return i;
                }
            }
            if (Debug.isDebugBuild) Debug.LogWarning("Quest Machine: Couldn't find ORK faction ID for faction named '" + factionName + "'");
            return 0;
        }

        #endregion

    }
}
